/*
Copyright 2006 Frank van der Loo

This file is part of Paratim.

Paratim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Paratim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Paratim; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "bomb.h"

// Returns the absolute value of a double
double dabs2(double value)
{
  if (value < 0.0)
    return value * -1.0;
  else
    return value;
 }

// Sets the x_direction and y_direction, which are the amount of pixels the bullet should change every step
void ratio2(double *x_direction, double *y_direction)
{
  if (*x_direction >= 0 && *y_direction >= 0)
  {
    if (*x_direction > *y_direction)
    {
      *x_direction = *x_direction / *y_direction;
      *y_direction = 1;
    }
    else
    {
      *y_direction = *y_direction / *x_direction;
      *x_direction = 1;
    }
  }
  else
  {
    if (dabs2(*x_direction) > dabs2(*y_direction))
    {
      *x_direction = *x_direction / *y_direction;
      *y_direction = 1;
    }
    else
    {
      *y_direction = dabs2(*y_direction) / dabs2(*x_direction);
      *x_direction = -1;
    }
  }
}

// bomb constructor, sets some default values
bomb::bomb() : sprite(DATADIR, "/bomb.bmp")
{
	#ifdef DEBUG
	  std::cout << "In bomb constructor\n";
	#endif
	dst.x = -20;
	dst.y = -20;
	dst.w = image->w;
	dst.h = image->h;
	black.w = image->w + 3;
	black.h = image->h + 3;
	black.x = dst.x - 3;
	black.y = dst.y - 3;	
	points = 30;
	end = false;
	#ifdef DEBUG
	  std::cout << "Einde bomb constructor\n";
	#endif
}

// initializes the bomb: sets the begin position and calculates to where it should drop
void bomb::init_bomb(int x, int y)
{
	dst.x = x;
	dst.y = y;
	doubley = dst.y;
	double dx = GUN_X - dst.x;
	double dy = GUN_Y - dst.y;
	ratio2(&dx, &dy);
	deriv = dy;
	//deriv = abs((GUN_Y - dst.y)/(GUN_X - x));
	#ifdef DEBUG
	std::cout << "x: " << dst.x << " y: " << dst.y << "\nderiv: " << deriv << std::endl;
	#endif
}

// moves the bomb the amount of pixels for one frame
void bomb::move()
{
	if (dst.x > 0 && dst.x < 800 && dst.y > 0 && dst.y < 600)	//CHANGE!!!!!!!!!!!!!!!!
	{
		if (dst.x < GUN_X)
			dst.x+=2;
		else
			dst.x-=2;
		doubley += (2*deriv); 
		dst.y=(int)doubley;
		//std::cout << "doubley: " << doubley << " dst.y: " << dst.y << std::endl;
		black.x = dst.x - 3;
		black.y = dst.y - 3;
	}
	else
		end = true;
}

// checks if the bomb hits the gun
bool bomb::hitsGun()
{
	if ((dst.x > GUN_X-5 && dst.x < GUN_X+5) && (dst.y > GUN_Y-5 && dst.y < GUN_Y+5))
	{
		std::cout << "\nHit by bomb!\n";
		return true;
	}
	return false;
}

// if the bomb is still there, move it
bool move_bomb(SDL_Surface *screen, bomb *plane_bomb)
{
	if (plane_bomb)
	{
		if (plane_bomb->hitsGun())
		{
			return true;
		}
		SDL_Rect bombblack = plane_bomb->getBlack();
		SDL_FillRect(screen, &bombblack, SDL_MapRGB(screen->format, 0, 0, 0));
		plane_bomb->move();
		SDL_Rect bombtemp = plane_bomb->getDstRect();
		SDL_BlitSurface(plane_bomb->getSurface(), NULL, screen, &bombtemp);
	}
	return false;
}
