/*
Copyright 2006 Frank van der Loo

This file is part of Paratim.

Paratim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Paratim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Paratim; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "SDL.h"
#include "bullet.h"
#include "llist.h"

bullet::bullet() : sprite(/*file*/DATADIR, "/bullet.bmp")
{
      dst.x = 390;
      dst.y = 380;
      ddstx = dst.x;
      ddsty = dst.y;
      dst.h = image->h;
      dst.w = image->w;
      
      black.w = image->w + 1;
      black.h = image->h + 1;
      black.x = dst.x - 1;
      black.y = dst.y - 1;
      end = false;
    }

// did the bullet hit a chopper to the left?
bool bullet::hits(chopperl *chop)
{
	SDL_Rect chptemp = chop->getDstRect();
	SDL_Rect buldsttemp = this->getDstRect();
	if (buldsttemp.y > chptemp.y && buldsttemp.y < chptemp.y+chptemp.h && buldsttemp.x > chptemp.x && buldsttemp.x < chptemp.x+chptemp.w)
	{
		return true;
	}
	return false;
}

// did the bullet hiet a chopper to the right?
bool bullet::hits(chopperr *chop)
{
	SDL_Rect chprtemp = chop->getDstRect();
	SDL_Rect buldsttemp = this->getDstRect();
	if (buldsttemp.y > chprtemp.y && buldsttemp.y < chprtemp.y+chprtemp.h && buldsttemp.x > chprtemp.x && buldsttemp.x < chprtemp.x+chprtemp.w)
	{
		return true;
	}
	return false;
}

// did the bullet hit a paratrooper?
int bullet::hits(trooper *troop)
{
	SDL_Rect trooptemp = troop->getDst2Rect();
	SDL_Rect trooptemp2 = troop->getDstRect();
	SDL_Rect buldsttemp = this->getDstRect();
	if (buldsttemp.y > trooptemp.y && buldsttemp.y < trooptemp.y+trooptemp.h && buldsttemp.x > trooptemp.x && buldsttemp.x < trooptemp.x+trooptemp.w)
	{
		return 1;		// trooper hit
	}
	if (buldsttemp.y > trooptemp2.y && buldsttemp.y < trooptemp2.y+trooptemp2.h && buldsttemp.x > trooptemp2.x && buldsttemp.x < trooptemp2.x+trooptemp2.w)
	{
		return 2;		// parachute hit
	}
	return 0;
}

// did the bullet hit a plane to the left?
bool bullet::hits(planel *airplanel)
{
	SDL_Rect planetemp = airplanel->getDstRect();
	SDL_Rect buldsttemp = this->getDstRect();
	if (buldsttemp.y > planetemp.y && buldsttemp.y < planetemp.y+planetemp.h && buldsttemp.x > planetemp.x && buldsttemp.x < planetemp.x+planetemp.w)
	{
		return true;
	}
	return false;
}

// did the bullet hit a plane to the right?
bool bullet::hits(planer *airplaner)
{
	SDL_Rect planetemp = airplaner->getDstRect();
	SDL_Rect buldsttemp = this->getDstRect();
	if (buldsttemp.y > planetemp.y && buldsttemp.y < planetemp.y+planetemp.h && buldsttemp.x > planetemp.x && buldsttemp.x < planetemp.x+planetemp.w)
	{
		return true;
	}
	return false;
}

// did the bullet hit a bomb?
bool bullet::hits(bomb *plane_bomb)
{
	SDL_Rect bombtemp = plane_bomb->getDstRect();
	SDL_Rect buldsttemp = this->getDstRect();
	if (buldsttemp.y > bombtemp.y && buldsttemp.y < bombtemp.y+bombtemp.h && buldsttemp.x > bombtemp.x && buldsttemp.x < bombtemp.x+bombtemp.w)
	{
		return true;
	}
	return false;
}

void bullet::move(double x_direction, double y_direction)
{
      black.y = dst.y - 1;
      black.x = dst.x - 1;
      if (dst.y < 0 || dst.x < 0 || dst.x > 800)
        end = true;
	if (x_direction == 0 || y_direction == 0)
	{
		ddstx += (x_direction*3);
      		ddsty -= (y_direction*3);
	}	
	else
	{
		ddstx += (x_direction*5);
      		ddsty -= (y_direction*5);
	}
      dst.y = (int) ddsty;
      dst.x = (int) ddstx;
      #ifdef DEBUG
      std::cout << "black.x: " << black.x << "\nblack.y: " << black.y << "\ndst.x: " << dst.x << "\ndst.y: " << dst.y << std::endl;
      #endif
    }

// if the bullet hits something, remove it and give points
int hit(SDL_Surface *screen, linkedList2 &troopLList, bullet *bultemp, chopperl *chopperlar[NUM_OF_CHOP], chopperr *chopperrar[NUM_OF_CHOP], planel **airplanel, planer **airplaner, bomb **plane_bomb)
{
  int points = 0;
  for (int k=0;k<troopLList.getSize();k++)
  {
	trooper *current_trooper = troopLList.get(k);
	if (current_trooper)
	{
		switch(bultemp->hits(current_trooper))
		{
			case 1:
			{
				bultemp->hit();
				current_trooper->hit();
				points += current_trooper->getPoints();
				SDL_Rect troopblack = current_trooper->getBlack();
				SDL_Rect troopblack2 = current_trooper->getBlack2();
				SDL_FillRect(screen, &troopblack, SDL_MapRGB(screen->format, 0, 0, 0));
				SDL_FillRect(screen, &troopblack2, SDL_MapRGB(screen->format, 0, 0, 0));
				//free(troopLList.get(k));
				free(current_trooper);
				troopLList.remove(k);
				//sdl_flip(screen);
				break;
			}
			case 2:
			{
				bultemp->hit();
				current_trooper->fall();
				points += current_trooper->getPoints();
				SDL_Rect troopblack = current_trooper->getBlack();
				SDL_FillRect(screen, &troopblack, SDL_MapRGB(screen->format, 0, 0, 0));
				//sdl_flip(screen);
				break;
			}
	  	}
        }
  }
      
  for (int j=0;j<NUM_OF_CHOP;j++)
  {
  	if (chopperlar[j])
	{
		if (bultemp->hits(chopperlar[j]))
		{
			points += chopperlar[j]->getPoints();
			bultemp->hit();
			chopperlar[j]->hit();
			SDL_Rect chopperblack = chopperlar[j]->getBlack();
			SDL_FillRect(screen, &chopperblack, SDL_MapRGB(screen->format, 0, 0, 0));
			free(chopperlar[j]);
			chopperlar[j] = NULL;
			//sdl_flip(screen);
		}
	}
	if (chopperrar[j])
	{
		if (bultemp->hits(chopperrar[j]))
		{
			points += chopperrar[j]->getPoints();
			bultemp->hit();
			chopperrar[j]->hit();
			SDL_Rect chopperblack = chopperrar[j]->getBlack();
			SDL_FillRect(screen, &chopperblack, SDL_MapRGB(screen->format, 0, 0, 0));
			free(chopperrar[j]);
			chopperrar[j] = NULL;
			//sdl_flip(screen);
		}
	}
  }
  if (*airplanel)
  {
  	if (bultemp->hits(*airplanel))
	{
		points += (*airplanel)->getPoints();
		bultemp->hit();
		(*airplanel)->hit();
		SDL_Rect planelblack = (*airplanel)->getBlack();
		SDL_FillRect(screen, &planelblack, SDL_MapRGB(screen->format, 0, 0, 0));
		free((*airplanel));
		(*airplanel) = NULL;
		//sdl_flip(screen);
	}
  }
  if (*airplaner)
  {
  	if (bultemp->hits(*airplaner))
	{
		points += (*airplaner)->getPoints();
		bultemp->hit();
		(*airplaner)->hit();
		SDL_Rect planerblack = (*airplaner)->getBlack();
		SDL_FillRect(screen, &planerblack, SDL_MapRGB(screen->format, 0, 0, 0));
		free((*airplaner));
		(*airplaner) = NULL;
		//sdl_flip(screen);
	}
  }
  if (*plane_bomb)
  {
  	if (bultemp->hits(*plane_bomb))
	{
		points += (*plane_bomb)->getPoints();
		bultemp->hit();
		(*plane_bomb)->hit();
		SDL_Rect bombblack = (*plane_bomb)->getBlack();
		SDL_FillRect(screen, &bombblack, SDL_MapRGB(screen->format, 0, 0, 0));
		free((*plane_bomb));
		(*plane_bomb) = NULL;
		//sdl_flip(screen);
	}
  }
  return points;
}

// moves the bullet
int move_bullet(SDL_Surface *screen, linkedList &LList, linkedList2 &troopLList, chopperl *chopperlar[NUM_OF_CHOP], chopperr *chopperrar[NUM_OF_CHOP], planel **airplanel, planer **airplaner, bomb **plane_bomb)
{
  int points = 0;
  bullet *bultemp = NULL;
  for (int i=0;i<LList.getSize();i++)
  {
    bultemp = LList.get(i);
    if (bultemp)
    {
      SDL_Rect buldsttemp = bultemp->getDstRect();
      SDL_Rect blacktemp = bultemp->getBlack();
      SDL_FillRect(screen, &blacktemp, SDL_MapRGB(screen->format, 0, 0, 0));
      bultemp->move(bultemp->get_x_dir(), bultemp->get_y_dir());
      SDL_BlitSurface(bultemp->getSurface(), NULL, screen, &buldsttemp);
      
      points += hit(screen, troopLList, bultemp, chopperlar, chopperrar, airplanel, airplaner, plane_bomb);
      //std::cout << "\nplanel: " << airplanel << "\nplaner: " << airplaner << std::endl;
      //std::cout << "\n*planel: " << *airplanel << "\n*planer: " << *airplaner << std::endl;
      
      if (bultemp->getEnd())
      {
        blacktemp = bultemp->getBlack();
	if (blacktemp.y < 0)
	  blacktemp.y = 0;
	if (blacktemp.x < 0)
	  blacktemp.x = 0;
	if (blacktemp.x > WIDTH-blacktemp.w)
	  blacktemp.x = WIDTH-blacktemp.w;
        free(bultemp);
        bultemp = NULL;
        LList.remove(i);
        SDL_FillRect(screen, &blacktemp, SDL_MapRGB(screen->format, 0, 0, 0));
      }
    }
  }
  return points;
}
