/*
Copyright 2006 Frank van der Loo

This file is part of Paratim.

Paratim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Paratim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Paratim; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "trooper.h"
#include "SDL.h"
#include "llist.h"

// creates a new paratrooper that starts at x,y
trooper::trooper(int x, int y) : sprite(DATADIR, "/parachute.bmp")
 {
   char *temppath = new char[strlen(DATADIR)+strlen("/paratrooper.bmp")+1];
   strcpy(temppath, DATADIR);
   temppath = strcat(temppath, "/paratrooper.bmp");
   image2 = SDL_LoadBMP(temppath);
   delete temppath;
   temppath = NULL;
   if (image2 == NULL)
   {
     std::cout << "Couldn't load data/paratrooper.bmp\n";
     exit(1);
   }
   SDL_SetColorKey(image2, SDL_SRCCOLORKEY, SDL_MapRGB(image2->format, 255, 255, 0));
   dst.x = x;
   dst.y = y;
   dst.w = image->w;
   dst.h = image->h;
   dst2.x = x;
   dst2.y = dst.y + image->h;
   dst2.w = image2->w;
   dst2.h = image2->h;
   black.w = image->w + 1;
   black.h = image->h + 3;
   black.x = dst.x - 1;
   black.y = dst.y - 2;
   black2.w = image2->w + 1;
   black2.h = image2->h + 3;
   black2.x = dst2.x - 1;
   black2.y = dst2.y - 2;
   points = 5;
   //end = false;
   //falling = false;
   state = FLOATING;
}

// moves the trooper a few fixels down
void trooper::move(SDL_Surface *screen, trooper* landed_troopers[])
{
  if (!this)
  {
  	std::cout << "this is NULL in regel 40!!!!\n";
	exit(1);
  }
  if (!image2)
  {
  	std::cout << "image2 is NULL in regel 45!!!!\n";
	exit(1);
  }
  if((dst2.y + image2->h) < 550)
  {
    dst.y+=2;
    dst2.y+=2;
    black.y+=2;
    black2.y+=2;
  }
  else
  {
  	//std::cout << (dst2.y + image2->h) << std::endl;
  	switch(state)
	{
		case FALLING:
		{
			drops_on_trooper(screen, this, landed_troopers);
			state = END;
			//end = true;
			break;
		}
		case FLOATING:
		{
			int i;
			for (i=0;i<20;i++)
			{
				if (landed_troopers[i] == NULL)
					break;
			}
			#ifdef DEBUG
			std::cout << "landed_value[" << i << "]" << std::endl;
			#endif
			landed_troopers[i] = this;
			state = LANDED;
		}
		case LANDED:
		case END:
		{}
	}
  }
}


// moves the troopers
void move_troopers(linkedList2 &troopLList, SDL_Surface *screen, trooper* landed_troopers[])
{
  for (int i=0;i<troopLList.getSize();i++)
  {
    trooper *troop = troopLList.get(i);
    if (troop)
    {
      SDL_Rect blacktrooptemp = troop->getBlack();
      SDL_Rect blacktrooptemp2 = troop->getBlack2();
      SDL_Rect trooptemp = troop->getDstRect();
      SDL_Rect troop2temp = troop->getDst2Rect();
      trooper_state state = troop->getState();
      if (state == END)
      {
        troopLList.remove(i);
	SDL_FillRect(screen, &blacktrooptemp2, SDL_MapRGB(screen->format, 0, 0, 0));
        //free(troopLList(i));			// MEMLEAK?!?!!?!
        //troopLList(i) = NULL;
        continue;
      }
      if (state == LANDED)
      {
        troopLList.remove_from_list(i);
	//concatenate(i);
      }
      SDL_FillRect(screen, &blacktrooptemp, SDL_MapRGB(screen->format, 0, 0, 0));
      SDL_FillRect(screen, &blacktrooptemp2, SDL_MapRGB(screen->format, 0, 0, 0));
      troop->move(screen, landed_troopers);
      if (state == FALLING)
        troop->move(screen, landed_troopers);
      if (state != FALLING)
        SDL_BlitSurface(troop->getSurface(), NULL, screen, &trooptemp);
      SDL_BlitSurface(troop->getSurface2(), NULL, screen, &troop2temp);
    }
    else
    {
      std::cout << "\nTrooper " << i << " is NULL!\n";
      exit(1);
    }
  }
}

// checks if the there are four troopers at one side of the gun
bool four_troopers(trooper* landed_troopers[])
{
	int left = 0, right = 0;
	if (landed_troopers[3] == NULL)
		return false;
	for (int i=0;i<20 && landed_troopers[i] != NULL;i++)
	{
		if ((landed_troopers[i]->getDstRect()).x < 350)
			left++;
		else
			right++;
	}
	if (left >= 4 || right >= 4)
	{
		std::cout << "4 troopers!\n";
		return true;
	}
	return false;
}

// concatenates the list of landed troopers when some trooper has been removed from it
void concatenate(int index, trooper* landed_troopers[])
{
	if (index == 19)
		return;
	for (int i=index;i<19;i++)
	{
		landed_troopers[i] = landed_troopers[i+1];
	}
}

// checks if the trooper "trooper" falls on top of a landed trooper
bool drops_on_trooper(SDL_Surface *screen, trooper* trooper, trooper* landed_troopers[])
{
	int trooper_x = (trooper->getDst2Rect()).x;
	int trooper2_x = trooper_x + (trooper->getDst2Rect()).w;
	for (int i=0;i<20 && landed_troopers[i] != NULL;i++)
	{
		if (((trooper_x > ((landed_troopers[i])->getDst2Rect()).x) && (trooper_x < ((landed_troopers[i])->getDst2Rect()).x + ((landed_troopers[i])->getDst2Rect()).w))
		|| ((trooper2_x > ((landed_troopers[i])->getDst2Rect()).x) && (trooper2_x < ((landed_troopers[i])->getDst2Rect()).x + ((landed_troopers[i])->getDst2Rect()).w)))
		{
			SDL_Rect black1 = (landed_troopers[i])->getBlack(), black2 = (landed_troopers[i])->getBlack2();;
			//SDL_FillRect(screen, &black1, SDL_MapRGB(screen->format, 0, 0, 0));
			//SDL_FillRect(screen, &black2, SDL_MapRGB(screen->format, 0, 0, 0));
			SDL_FillRect(screen, &black1, SDL_MapRGB(screen->format, 0, 0, 0));
			SDL_FillRect(screen, &black2, SDL_MapRGB(screen->format, 0, 0, 0));
			free(landed_troopers[i]);
			//landed_troopers[i] = NULL;
			concatenate(i, landed_troopers);
			//SDL_Flip(screen);
			#ifdef DEBUG
			std::cout << "DEAD!\n";
			#endif
			//exit(1);
		}
	}
	return false;
}
